///***Matei Andra Gabriela***///
///***     Grupa 331CC    ***///
///***     Proiect PM     ***///
///***   Interactive Toy  ***///
#include <avr/io.h>
#include <string.h>
#include <stdio.h>
#include "diskio.h"
#include "pff.h"
#include "servomotor.c"

/* Four-Character Code - folosit pentru a indentifica formate de date */
#define FCC(c1, c2, c3, c4) \
	(((DWORD)(c4) << 24) + \
	 ((DWORD)(c3) << 16) + \
	 (( WORD)(c2) <<  8) + \
	 (( BYTE)(c1) <<  0))

unsigned char Buff1[256];										
unsigned char Buff2[256];										
volatile uint8_t read1 = 1;
volatile uint16_t counter = 0;
volatile uint8_t sync = 0;
volatile BYTE FifoRi, FifoWi, FifoCt;
FATFS Fs;													
DIR Dir;

/* File information */
FILINFO Fno;
/* PA5 = buton pentru salut*/													
BOOL b1_pressed, 
/* PA6 = buton pentru led */
	 b2_pressed, 
/* PA7 = buton pentru servomotor */
	 b3_pressed; 								

int wait1, wait2;

/*Incarcare header pentru un fisier .wav*/
DWORD load_header(void)
{
	DWORD size;
	WORD ret;

	// citeste header-ul (12 octeti)
	if(pf_read(BUF_FRONT, 12, &ret))
		return 1;

	if(ret != 12 || LD_DWORD(BUF_FRONT + 8) != FCC('W','A','V','E'))
		return 0;

	for(;;)
	{
		// citeste chunk ID si size
		pf_read(BUF_FRONT, 8, &ret);
		if(ret != 8)
			return 0;

		size = LD_DWORD(&BUF_FRONT[4]);

		// verifica FCC
		switch(LD_DWORD(&BUF_FRONT[0]))
		{
			// 'fmt ' chunk
			case FCC('f','m','t',' '):
				// verifica size
				if(size > 100 || size < 16) return 0;

				// citeste continutul
				pf_read(BUF_FRONT, size, &ret);
				// verifica codificarea
				if(ret != size || BUF_FRONT[0] != 1) return 0;
				// verifica numarul de canale
				if(BUF_FRONT[2] != 1 && BUF_FRONT[2] != 2) return 0;
				// verifica rezolutia
				if(BUF_FRONT[14] != 8 && BUF_FRONT[14] != 16) return 0;

				// seteaza sampling rate-ul
				OCR0A = (BYTE)(F_CPU / 8 / LD_WORD(&BUF_FRONT[4])) - 1;
				break;

			// 'data' chunk => incepe redarea
			case FCC('d','a','t','a'):
				return size;

			// 'LIST' chunk => skip
			case FCC('L','I','S','T'):
			// 'fact' chunk => skip
			case FCC('f','a','c','t'):
				pf_lseek(fs.fptr + size);
				break;

			// chunk necunoscut => eroare
			default:
				return 0;
		}
	}

	return 0;
}



ISR(TIMER0_COMPA_vect)
{
	OCR1A = BUF_FRONT[buf_pos++];

	// swap buffers when end is reached (end is 256 <=> overflow to 0)
	if(buf_pos == 0)
		buf_front = 1 - buf_front;
}

void Timer0_init(void) 
{
	TCCR0A = _BV(WGM01);
	TCCR0B = _BV(CS01);											/* CTC, FCPU/8 */
	TIMSK0 = _BV(OCIE0A);
}

void Timer1_init(void) 
{
	TCCR1A = _BV(COM1A1)|_BV(COM1A0)|_BV(WGM10);
	TCCR1B = _BV(WGM12)|_BV(CS10);
}

void reset_state(void) 
{
	b1_pressed = FALSE;
	b2_pressed = FALSE;
	b3_pressed = FALSE;
		
	state = 0;
}

/* Functie redare fisier .wav */
UINT play(const char *path)
{
	FRESULT ret;

	if((ret = pf_open(path)) == FR_OK)
	{
		WORD bytes_read;

		// incarca header-ul fisierului
		DWORD current_size = load_header();
		if(current_size < buf_size)
			return FR_NO_FILE;

		// align to sector boundary
		ret = pf_lseek((fs.fptr + 511) & ~511);
		if(ret != FR_OK)
			return ret;

		// fill front buffer
		ret = pf_read(BUF_FRONT, buf_size, &bytes_read);
		if(ret != FR_OK)
			return ret;
		if(bytes_read < buf_size)
			return ret;

		// reset front buffer index
		buf_pos = 0;

		// start output
		timer0_start();
		timer1_start();
		DDRD |= (1 << PD5);

		while(continue_play())
		{
			uint8_t old_buf_front = buf_front;
			
			// fill back buffer
			ret = pf_read(BUF_BACK, buf_size, &bytes_read);
			if(ret != FR_OK)
				break;
			if(bytes_read < buf_size)
				break;

			// wait for buffer swap
			while(old_buf_front == buf_front);
		}

		// stop output
		DDRD &= ~(1 << PD5);
		timer1_stop();
		timer0_stop();
	}

	return ret;
}

void get_music(const char *path)
{
	if(found_files > 0)
		return;

	DIR dir;
	if(pf_opendir(&dir, path) != FR_OK)
	{
		LCD_printAt(0x00, "music missing");
		return;
	}

	FILINFO file;
	while(found_files < 2 && pf_readdir(&dir, &file) == FR_OK &&
		file.fname != NULL && file.fname[0] != '\0')
	{
		if((file.fattrib & (AM_DIR | AM_HID)) == 0 &&
			strstr(file.fname, ".WAV") != NULL)
		{
			snprintf(music_files[found_files], sizeof(music_files[found_files]), "%s/%s", path, file.fname);
			found_files++;
		}
	}
}

void toy (void)
{
	if (!(PINA & (1 << PA6))){
		play("Hello.wav");
		b2_pressed = TRUE;
	}
	else if (!(PINA & (1 << PA7))){
		b3_pressed = TRUE;
	}
	if (b2_pressed == TRUE || b3_pressed == TRUE){
		servo();
	}
}

int main()
{
	/* Setare pini si porturi */
	DDRA |= (1 << PA0) | (1 << PA1);
	DDRA &= ~(1 << PA2) & ~(1 << PA3) & ~(1 << PA4) & ~(1 << PA5) & ~(1 << PA6) & ~(1 << PA7);
	PORTA |= (1 << PA2) | (1 << PA3) | (1 << PA4) | (1 << PA5) | (1 << PA6) | (1 << PA7);

	DDRB |= (1 << PB4) | (1 << PB5) | (1 << PB7);
	PORTB &= ~(1 << PB6);

	DDRC = 0xFF;
	DDRD = 0xFF;

	//PORTA ^= ~(0x01 << PA0);
	//PORTA ^= ~(0x01 << PA1);
	
	
	for (;;) {
		_delay_ms(100);
		PORTD ^= ~(0x01 << 5);
		if (pf_mount(&Fs)) {	
			continue;							
		}
		
		if (pf_opendir(&Dir, "Toy"))
		{
			PORTA = PORTA ^ (0x01 << PA0);
			break;
		}
		while(1){
			toy();
		}	
	}
	return 1;
}